import asynctest
from asynctest.mock import MagicMock
from asynctest.mock import call


class TestSlackConnectionStreamMessages(asynctest.TestCase):

    def setUp(self):
        from charlesbot.slack.slack_connection import SlackConnection
        self.slack = SlackConnection()
        self.slack.initialized = True
        self.slack.sc = MagicMock()
        self.slack.connect = MagicMock()

    def tearDown(self):
        self.slack._drop()

    def test_stream_none(self):
        self.slack.sc.rtm_read.return_value = []
        messages = yield from self.slack.get_stream_messages()
        self.assertEqual(messages, [])

    def test_stream_error(self):
        self.slack.sc.rtm_read.side_effect = [BrokenPipeError]
        with self.assertRaises(SystemExit):
            yield from self.slack.get_stream_messages()

    def test_timeout_error(self):
        self.slack.sc.rtm_read.side_effect = [TimeoutError]
        yield from self.slack.get_stream_messages()
        self.assertEqual(self.slack.connect.mock_calls, [call()])

    def test_stream_one_message(self):
        self.slack.sc.rtm_read.return_value = ["boo"]
        messages = yield from self.slack.get_stream_messages()
        self.assertEqual(messages, [None])

    def test_stream_two_messages(self):
        self.slack.sc.rtm_read.return_value = ["boo", "urns"]
        messages = yield from self.slack.get_stream_messages()
        self.assertEqual(messages, [None, None])

    def test_stream_channel_joined(self):
        self.slack.sc.rtm_read.return_value = [{"type": "channel_joined"}]
        messages = yield from self.slack.get_stream_messages()
        self.assertEqual(len(messages), 1)
        self.assertEqual("", messages[0].is_channel)

    def test_stream_channel_left(self):
        self.slack.sc.rtm_read.return_value = [{"type": "channel_left"}]
        messages = yield from self.slack.get_stream_messages()
        self.assertEqual(len(messages), 1)
        self.assertEqual("", messages[0].channel)

    def test_stream_group_joined(self):
        self.slack.sc.rtm_read.return_value = [{"type": "group_joined"}]
        messages = yield from self.slack.get_stream_messages()
        self.assertEqual(len(messages), 1)
        self.assertEqual("", messages[0].is_group)

    def test_stream_group_left(self):
        self.slack.sc.rtm_read.return_value = [{"type": "group_left"}]
        messages = yield from self.slack.get_stream_messages()
        self.assertEqual(len(messages), 1)
        self.assertEqual("", messages[0].channel)

    def test_stream_message(self):
        self.slack.sc.rtm_read.return_value = [{"type": "message"}]
        messages = yield from self.slack.get_stream_messages()
        self.assertEqual(len(messages), 1)
        self.assertEqual("", messages[0].subtype)



from abc import ABCMeta, abstractmethod
from charlesbot.slack.slack_message import SlackMessage
from charlesbot.slack.slack_connection import SlackConnection
import asyncio
import logging


class BasePlugin(metaclass=ABCMeta):

    def __init__(self, plugin_name):
        self.set_running(True)
        self.log = logging.getLogger(__name__)
        self.slack = SlackConnection()
        self._plugin_name = plugin_name
        self.log.info("Initializing the %s plugin" % plugin_name)
        self._q = asyncio.Queue()
        self.initialize_queue_consumer()

    def initialize_queue_consumer(self):
        loop = asyncio.get_event_loop()
        loop.create_task(self.consume())

    @asyncio.coroutine
    def consume(self):
        while self.is_running():
            value = yield from self._q.get()
            if self.is_bot_message(value):
                continue
            yield from self.process_message(value)

    def is_bot_message(self, message):
        if not type(message) is SlackMessage:
            return False
        if message.subtype == "bot_message":
            return True
        return False

    def get_plugin_name(self):
        return self._plugin_name

    def is_running(self):
        return self._is_running

    def set_running(self, running):
        self._is_running = running

    @asyncio.coroutine
    def queue_message(self, message):
        yield from self._q.put(message)

    @abstractmethod
    def get_help_message(self):
        pass

    @abstractmethod
    def process_message(self, message):
        pass



from charlesbot.base_plugin import BasePlugin
from charlesbot.util.parse import does_msg_contain_prefix
from charlesbot.slack.slack_message import SlackMessage
import asyncio


class Help(BasePlugin):

    def __init__(self):
        super().__init__("Help!")
        self.initialize_help_message_list()

    def initialize_help_message_list(self):
        self.help_msg_list = []
        self.help_msg_list.append("!help - This help message")
        self.help_msg_list.sort()

    def add_help_message(self, message):
        if message:
            self.help_msg_list.append(str(message))
            self.help_msg_list.sort()

    def get_help_message(self):
        help_msg = "\n".join(self.help_msg_list)
        return "```\n%s\n```" % help_msg

    @asyncio.coroutine
    def process_message(self, message):
        if not type(message) is SlackMessage:
            return
        if does_msg_contain_prefix("!help", message.text):
            yield from self.send_help_message(message.channel)

    @asyncio.coroutine
    def send_help_message(self, channel_id):  # pragma: no cover
        yield from self.slack.send_channel_message(channel_id,
                                                   self.get_help_message())



from charlesbot.slack.slack_room_joined import SlackRoomJoined


class SlackGroupJoined(SlackRoomJoined):

    properties = ['members',
                  'is_group',
                  'is_archived',
                  'creator',
                  'id',
                  'name',
                  'type']

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    @property
    def compatibility_key(self):
        return "group_joined"



import json
import asyncio
from charlesbot.base_object import BaseObject


class SlackUser(BaseObject):

    main_properties = ['id',
                       'name',
                       'deleted',
                       'color',
                       'is_admin',
                       'is_owner',
                       'has_2fa',
                       'is_bot']

    profile_properties = ['first_name',
                          'last_name',
                          'real_name',
                          'email',
                          'image_24']

    properties = main_properties + profile_properties

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    @asyncio.coroutine
    def retrieve_slack_user_info(self, slack_connection, user_id):
        result = yield from slack_connection.api_call('users.info',
                                                      user=user_id)
        self.load(json.loads(result))

    def load(self, user_dict):
        for prop in self.main_properties:
            default = getattr(self, prop)
            setattr(self, prop, user_dict.get('user', {}).get(prop, default))  # NOQA

        for prop in self.profile_properties:
            default = getattr(self, prop)
            setattr(self, prop, user_dict.get('user', {}).get('profile', {}).get(prop, default))  # NOQA



import unittest
from charlesbot.util.slack import get_robot_channel_membership
from tests import load_fixture


class TestGetRobotChannelMembership(unittest.TestCase):

    def setUp(self):
        self.channel_list_one = load_fixture('channel_list_one.json')
        self.channel_list_two = load_fixture('channel_list_two.json')
        self.channel_list_three = load_fixture('channel_list_three.json')
        self.channel_list_four = load_fixture('channel_list_four.json')
        self.channel_list_five = load_fixture('channel_list_five.json')
        self.channel_list_six = load_fixture('channel_list_six.json')
        self.channel_list_seven = load_fixture('channel_list_seven.json')
        self.channel_list_eight = load_fixture('channel_list_eight.json')
        self.channel_list_nine = load_fixture('channel_list_nine.json')
        self.channel_list_ten = load_fixture('channel_list_ten.json')

    def test_slack_channel_membership_single(self):
        channel_list = self.channel_list_one
        membership = get_robot_channel_membership(channel_list)
        self.assertEqual(len(membership), 1)
        self.assertCountEqual(membership, {'C024BE91L': 'fun1'})

    def test_slack_channel_membership_none(self):
        channel_list = self.channel_list_three
        membership = get_robot_channel_membership(channel_list)
        self.assertEqual(len(membership), 0)
        self.assertCountEqual(membership, {})

    def test_slack_channel_membership_error_1(self):
        # malformed json
        channel_list = self.channel_list_four
        with self.assertRaises(ValueError):
            get_robot_channel_membership(channel_list)

    def test_slack_channel_membership_error_2(self):
        # channel key not present in json file
        channel_list = self.channel_list_five
        with self.assertRaises(KeyError):
            get_robot_channel_membership(channel_list)

    def test_slack_channel_membership_multiple_0(self):
        # member of none of three channels
        channel_list = self.channel_list_six
        membership = get_robot_channel_membership(channel_list)
        self.assertEqual(len(membership), 0)
        self.assertCountEqual(membership, {})

    def test_slack_channel_membership_multiple_1(self):
        # member of one of three channels
        channel_list = self.channel_list_seven
        membership = get_robot_channel_membership(channel_list)
        self.assertEqual(len(membership), 1)
        self.assertCountEqual(membership, {'C024BE91B': 'fun2'})

    def test_slack_channel_membership_multiple_2(self):
        # member of two of three channels
        channel_list = self.channel_list_eight
        membership = get_robot_channel_membership(channel_list)
        self.assertEqual(len(membership), 2)
        self.assertCountEqual(
            membership,
            {'C024BE91B': 'fun2', 'C024BE91C': 'fun3'}
        )

    def test_slack_channel_membership_multiple_3(self):
        # member of three of three channels
        channel_list = self.channel_list_nine
        membership = get_robot_channel_membership(channel_list)
        self.assertEqual(len(membership), 3)
        self.assertCountEqual(
            membership,
            {'C024BE91B': 'fun2', 'C024BE91C': 'fun3', 'C024BE91A': 'fun1'}
        )

    def test_slack_channel_membership_general(self):
        # general channel
        channel_list = self.channel_list_ten
        membership = get_robot_channel_membership(channel_list)
        self.assertEqual(len(membership), 1)
        self.assertCountEqual(
            membership,
            {'C024BE91B': 'fun2'}
        )



import asynctest
from asynctest.mock import MagicMock
from tests import load_fixture
import json


class TestSlackConnectionApiCall(asynctest.TestCase):

    def setUp(self):
        self.channel_list_one = load_fixture('channel_list_one.json')
        self.channel_list_two = load_fixture('channel_list_two.json')
        self.channel_list_three = load_fixture('channel_list_three.json')
        self.channel_list_four = load_fixture('channel_list_four.json')

        from charlesbot.slack.slack_connection import SlackConnection
        self.slack = SlackConnection()
        self.slack.initialized = True
        self.slack.sc = MagicMock()

    def tearDown(self):
        self.slack._drop()

    def test_slack_rtm_api_call_ok(self):
        channel_list = self.channel_list_one.encode('utf8')
        self.slack.sc.api_call.return_value = channel_list
        val = yield from self.slack.api_call("fake_endpoint")
        self.assertEqual(
            json.loads(val),
            json.loads(channel_list.decode('utf-8'))
        )

    def test_slack_rtm_api_call_not_ok(self):
        channel_list = self.channel_list_two.encode('utf8')
        self.slack.sc.api_call.return_value = channel_list
        val = yield from self.slack.api_call("fake_endpoint")
        self.assertEqual(json.loads(val), '{}')

    def test_slack_rtm_api_call_not_encoded_utf8(self):
        channel_list = self.channel_list_three
        self.slack.sc.api_call.return_value = channel_list
        with self.assertRaises(AttributeError):
            yield from self.slack.api_call("fake_endpoint")

    def test_slack_rtm_api_call_malformed_json(self):
        channel_list = self.channel_list_four.encode('utf8')
        self.slack.sc.api_call.return_value = channel_list
        with self.assertRaises(ValueError):
            yield from self.slack.api_call("fake_endpoint")



import unittest
from charlesbot.slack.slack_base_object import SlackBaseObject


class TestSlackBaseObject(unittest.TestCase):

    class DummyObject(SlackBaseObject):

        properties = [
            "property1",
            "property2",
            "type",
        ]

        def __init__(myself, **kwargs):
            super().__init__(**kwargs)

        @property
        def compatibility_key(self):
            return "dummymessage"

    def setUp(self):
        self.dc = TestSlackBaseObject.DummyObject()

    def test_compatibility_type_not_present(self):
        object_dict = {}
        self.assertFalse(self.dc.is_compatible(object_dict))  # NOQA

    def test_compatibility_type_empty(self):
        object_dict = {
            "type": "",
        }
        self.assertFalse(self.dc.is_compatible(object_dict))  # NOQA

    def test_compatibility_type_invalid(self):
        object_dict = {
            "type": "boo",
        }
        self.assertFalse(self.dc.is_compatible(object_dict))  # NOQA

    def test_compatibility_valid(self):
        object_dict = {
            "type": "dummymessage",
        }
        self.assertTrue(self.dc.is_compatible(object_dict))  # NOQA



import re


def parse_msg_with_prefix(prefix, msg):
    regex_str = "^\s*%s[ :](.*)" % prefix
    p = re.compile(regex_str, re.IGNORECASE)
    m = p.match(msg)
    if m:
        return m.group(1).strip()
    else:
        return None


def does_msg_contain_prefix(prefix, msg):
    regex_str = "^\s*%s(\s+\S+.*|\s+)?$" % prefix
    p = re.compile(regex_str, re.IGNORECASE)
    m = p.match(msg)
    if m:
        return True
    else:
        return False



import unittest
from charlesbot.util.slack import get_robot_info
import json


class TestGetRobotInfo(unittest.TestCase):

    def test_robot_info_ok(self):
        auth_test = {
            "ok": True,
            "url": "https:\/\/myteam.slack.com\/",
            "team": "My Team",
            "user": "cal",
            "team_id": "T12345",
            "user_id": "U12345"
        }
        retval = get_robot_info(json.dumps(auth_test))
        self.assertEqual(retval, {"U12345": "cal"})

    def test_robot_info_missing(self):
        auth_test = {
            "ok": True,
            "url": "https:\/\/myteam.slack.com\/",
            "team": "My Team",
            "team_id": "T12345",
        }
        with self.assertRaises(KeyError):
            get_robot_info(json.dumps(auth_test))

    def test_robot_info_missing_user(self):
        auth_test = {
            "ok": True,
            "url": "https:\/\/myteam.slack.com\/",
            "team": "My Team",
            "team_id": "T12345",
            "user_id": "U12345"
        }
        with self.assertRaises(KeyError):
            get_robot_info(json.dumps(auth_test))

    def test_robot_info_missing_user_id(self):
        auth_test = {
            "ok": True,
            "url": "https:\/\/myteam.slack.com\/",
            "team": "My Team",
            "user": "cal",
            "team_id": "T12345",
        }
        with self.assertRaises(KeyError):
            get_robot_info(json.dumps(auth_test))



import json
import logging
log = logging.getLogger(__name__)


def get_robot_info(auth_test):
    json_str = json.loads(auth_test)
    robot_name = json_str["user"]
    robot_id = json_str["user_id"]
    return {robot_id: robot_name}


def get_robot_channel_membership(channel_list):
    json_list = json.loads(channel_list)
    membership = {}
    for channel in json_list['channels']:
        if channel['is_member'] and not channel['is_general']:
            channel_id = channel['id']
            channel_name = channel['name']
            membership[channel_id] = channel_name
    return membership


def get_robot_group_membership(group_list):
    json_list = json.loads(group_list)
    membership = {}

    if 'groups' not in json_list:
        return membership

    for group in json_list['groups']:
        group_id = group['id']
        group_name = group['name']
        membership[group_id] = group_name
    return membership



import unittest
import io
import yaml
import os
from unittest.mock import mock_open
from unittest.mock import patch
from unittest.mock import call


class TestConfiguration(unittest.TestCase):

    def test_read_config_dict_valid(self):
        m = mock_open()
        yaml_file = """
        main:
          slackbot_token: 'token123'
          enabled_plugins:
            - none
        """
        with patch('charlesbot.config.configuration.open',
                   m,
                   create=True):
            m.return_value = io.StringIO(yaml_file)
            from charlesbot.config import configuration
            result = configuration.read_config_dict("fake filename")
            m.assert_has_calls([call("fake filename", "r")])
            self.assertTrue(result['main']['slackbot_token'], 'token123')
            self.assertTrue(result['main']['enabled_plugins'], ['none'])

    def test_read_config_dict_invalid(self):
        m = mock_open()
        yaml_file = "{:"
        with patch('charlesbot.config.configuration.open',
                   m,
                   create=True):
            m.return_value = io.StringIO(yaml_file)
            from charlesbot.config import configuration
            with self.assertRaises(yaml.parser.ParserError):
                configuration.read_config_dict("fake filename")
            m.assert_has_calls([call("fake filename", "r")])

    @patch.dict('os.environ', {'CHARLESBOT_SETTINGS_FILE': 'filename'})
    def test_config_file_name(self):
        from charlesbot.config import configuration
        self.assertEqual(configuration.get_config_file_name(), 'filename')

    @patch.dict('os.environ', {'CHARLESBOT_SETTINGS_FILE': ''})
    def test_config_file_name_empty(self):
        from charlesbot.config import configuration
        self.assertEqual(configuration.get_config_file_name(),
                         './development.yaml')

    @patch.dict('os.environ', {'CHARLESBOT_SETTINGS_FILE': 'filename'})
    def test_config_file_name_not_present(self):
        from charlesbot.config import configuration
        del os.environ['CHARLESBOT_SETTINGS_FILE']
        self.assertEqual(configuration.get_config_file_name(),
                         './development.yaml')

    def test_get(self):
        with patch('charlesbot.config.configuration.read_config_dict') as mock_read_dict:  # NOQA
            from charlesbot.config import configuration
            mock_read_dict.return_value = {"test": "one"}
            result = configuration.get()
            self.assertEqual(result, {"test": "one"})



from abc import ABCMeta, abstractproperty
from charlesbot.base_object import BaseObject


class SlackBaseObject(BaseObject, metaclass=ABCMeta):

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    @abstractproperty
    def compatibility_key(self):
        pass

    def is_compatible(self, object_dict):
        try:
            my_type = object_dict.get('type', "")
            return my_type == self.compatibility_key
        except AttributeError:
            return False



import unittest
import json
from charlesbot.slack.slack_attachment import SlackAttachment


class TestSlackAttachment(unittest.TestCase):

    def test_attachment_equality(self):
        attach1 = SlackAttachment(color="red",
                                  fallback="fallback msg",
                                  text="real message")
        attach2 = SlackAttachment(color="blue",
                                  fallback="fallback msg 2",
                                  text="real message 2")
        self.assertNotEqual(attach1, attach2)
        attach_dict = {
            "color": "red"
        }
        attach2.load(attach_dict)
        self.assertNotEqual(attach1, attach2)
        attach_dict = {
            "fallback": "fallback msg"
        }
        attach2.load(attach_dict)
        self.assertNotEqual(attach1, attach2)
        attach_dict = {
            "text": "real message"
        }
        attach2.load(attach_dict)
        self.assertEqual(attach1, attach2)

    def test_attachment_string(self):
        attach1 = SlackAttachment(color="green",
                                  fallback="fallback msg green",
                                  text="real message green")
        dummy_json = json.loads(str(attach1))
        self.assertEqual(len(dummy_json), 1)
        self.assertEqual(len(dummy_json[0].keys()), 9)
        self.assertEqual(dummy_json[0].get('color'), "green")
        self.assertEqual(dummy_json[0].get('fallback'), "fallback msg green")
        self.assertEqual(dummy_json[0].get('text'), "real message green")



import unittest
from charlesbot.slack.slack_channel_joined import SlackChannelJoined
from charlesbot.slack.slack_channel_left import SlackChannelLeft
from charlesbot.slack.slack_group_joined import SlackGroupJoined
from charlesbot.slack.slack_group_left import SlackGroupLeft
from charlesbot.slack.slack_message import SlackMessage


class TestSlackBaseObjectChildren(unittest.TestCase):

    def test_slack_channel_joined_compatibility(self):
        sc = SlackChannelJoined()
        object_dict = {"type": "channel_joined"}
        self.assertTrue(sc.is_compatible(object_dict))

    def test_slack_channel_left_compatibility(self):
        sc = SlackChannelLeft()
        object_dict = {"type": "channel_left"}
        self.assertTrue(sc.is_compatible(object_dict))

    def test_slack_group_joined_compatibility(self):
        sc = SlackGroupJoined()
        object_dict = {"type": "group_joined"}
        self.assertTrue(sc.is_compatible(object_dict))

    def test_slack_group_left_compatibility(self):
        sc = SlackGroupLeft()
        object_dict = {"type": "group_left"}
        self.assertTrue(sc.is_compatible(object_dict))

    def test_slack_message_compatibility(self):
        sc = SlackMessage()
        object_dict = {"type": "message"}
        self.assertTrue(sc.is_compatible(object_dict))



from charlesbot.slack.slack_base_object import SlackBaseObject


class SlackChannelLeft(SlackBaseObject):

    properties = ['channel',
                  'type']

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    @property
    def compatibility_key(self):
        return "channel_left"



import logging
import sys
from charlesbot.robot import Robot


def main(args=None):
    logging.basicConfig(
        stream=sys.stdout,
        level=logging.INFO,
        format='%(asctime)s: %(levelname)s [%(name)s:%(lineno)d] %(message)s'
    )
    log = logging.getLogger(__name__)
    log.info("Starting CharlesBOT now!")

    slackbot = Robot()
    slackbot.start()

if __name__ == "__main__":
    main()



import unittest
from charlesbot.slack.slack_room_joined import SlackRoomJoined
import json


class TestSlackRoomJoined(unittest.TestCase):

    class DummyObject(SlackRoomJoined):

        properties = [
            "property1",
            "property2",
            "type",
        ]

        def __init__(myself, **kwargs):
            super().__init__(**kwargs)

        @property
        def compatibility_key(self):
            return "dummykey"

    def setUp(self):
        self.dc = TestSlackRoomJoined.DummyObject()

    def test_compatibility(self):
        self.assertTrue(self.dc.is_compatible({"type": "dummykey"}))

    def test_add_extra_property(self):
        property_dict = {
            "batman": "robin",
            "type": "such type",
        }
        self.dc.load(property_dict)
        dummy_json = json.loads(str(self.dc))
        self.assertEqual(len(dummy_json.keys()), 3)
        self.assertEqual(dummy_json.get('property1'), "")
        self.assertEqual(dummy_json.get('property2'), "")
        self.assertEqual(dummy_json.get('type'), "such type")
        self.assertEqual(dummy_json.get('batman', "default"), "default")

    def test_update_property_one(self):
        property_dict = {
            "type": "such type",
            "channel": {
                "property1": "property1 value",
            }
        }
        self.dc.load(property_dict)
        dummy_json = json.loads(str(self.dc))
        self.assertEqual(len(dummy_json.keys()), 3)
        self.assertEqual(dummy_json.get('property1'), "property1 value")
        self.assertEqual(dummy_json.get('property2'), "")
        self.assertEqual(dummy_json.get('type'), "such type")

    def test_update_property_two(self):
        property_dict = {
            "type": "such type",
            "channel": {
                "property2": "property2 value",
            }
        }
        self.dc.load(property_dict)
        dummy_json = json.loads(str(self.dc))
        self.assertEqual(len(dummy_json.keys()), 3)
        self.assertEqual(dummy_json.get('property1'), "")
        self.assertEqual(dummy_json.get('property2'), "property2 value")
        self.assertEqual(dummy_json.get('type'), "such type")

    def test_update_properties(self):
        property_dict = {
            "type": "such type",
            "channel": {
                "property1": "property1 value",
                "property2": "property2 value",
            }
        }
        self.dc.load(property_dict)
        dummy_json = json.loads(str(self.dc))
        self.assertEqual(len(dummy_json.keys()), 3)
        self.assertEqual(dummy_json.get('property1'), "property1 value")
        self.assertEqual(dummy_json.get('property2'), "property2 value")
        self.assertEqual(dummy_json.get('type'), "such type")



from abc import ABCMeta
import logging
import json


class BaseObject(metaclass=ABCMeta):

    def __init__(self, **kwargs):
        self.log = logging.getLogger(__name__)
        for prop in self.properties:
            setattr(self, prop, kwargs.get(prop, ""))

    def load(self, object_dict):
        for prop in self.properties:
            default = getattr(self, prop)
            setattr(self, prop, object_dict.get(prop, default))

    def __str__(self):
        return_dict = {}
        for prop in self.properties:
            return_dict.update({prop: getattr(self, prop)})
        return json.dumps(return_dict)

    def __eq__(self, other):
        for prop in self.properties:
            if not getattr(self, prop) == getattr(other, prop):
                self.log.debug("Property %s is different" % prop)
                self.log.debug("%s != %s" % (getattr(self, prop),
                                             getattr(other, prop)))
                return False
        return True

    def __ne__(self, other):
        return not self.__eq__(other)



from abc import ABCMeta, abstractproperty
from charlesbot.slack.slack_base_object import SlackBaseObject


class SlackRoomJoined(SlackBaseObject, metaclass=ABCMeta):

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    @abstractproperty
    def compatibility_key(self):
        pass

    def load(self, object_dict):
        for prop in self.properties:
            default = getattr(self, prop)
            setattr(self, prop, object_dict.get('channel', {}).get(prop, default))  # NOQA
        self.type = object_dict.get('type')



import asyncio
import aiohttp
import logging
log = logging.getLogger(__name__)


@asyncio.coroutine
def http_get_auth_request(auth_string,
                          url,
                          content_type="application/json",
                          auth_method="Token",
                          payload={}):
    headers = {
        'Content-type': content_type,
        'Authorization': "%s %s" % (auth_method, auth_string),
    }
    response = yield from aiohttp.get(url, headers=headers, params=payload)
    if not response.status == 200:
        text = yield from response.text()
        log.error("URL: %s" % url)
        log.error("Response status code was %s" % str(response.status))
        log.error(response.headers)
        log.error(text)
        response.close()
        return ""
    return (yield from response.text())


@asyncio.coroutine
def http_get_request(url, content_type="application/json"):
    headers = {
        'Content-type': content_type,
    }
    response = yield from aiohttp.get(url, headers=headers)
    if not response.status == 200:
        text = yield from response.text()
        log.error("URL: %s" % url)
        log.error("Response status code was %s" % str(response.status))
        log.error(response.headers)
        log.error(text)
        response.close()
        return ""
    return (yield from response.text())



import asynctest
from asynctest.mock import MagicMock
from asynctest.mock import call
from asynctest.mock import patch
from charlesbot.slack.slack_message import SlackMessage
from charlesbot.slack.slack_pong import SlackPong


class TestPingPlugin(asynctest.TestCase):

    def setUp(self):
        patcher1 = patch('charlesbot.plugins.ping_plugin.Ping.schedule_ping_frequency')  # NOQA
        self.addCleanup(patcher1.stop)
        self.mock_schedule_ping_frequency = patcher1.start()

        patcher2 = patch('charlesbot.plugins.ping_plugin.Ping.get_package_version_number')  # NOQA
        self.addCleanup(patcher2.stop)
        self.mock_get_package_version_number = patcher2.start()

        from charlesbot.plugins.ping_plugin import Ping
        self.ping = Ping()
        self.ping.send_version_message = MagicMock()

    @asynctest.ignore_loop
    def test_get_help_string(self):
        """
        Don't care about the contents of the help string, just as long as there
        is something in there.
        """
        help_str = self.ping.get_help_message()
        self.assertTrue(help_str)

    def test_process_incorrect_msg_type(self):
        yield from self.ping.process_message("invalid")
        self.assertEqual(self.ping.send_version_message.mock_calls, [])

    def test_process_irrelevant_slack_message(self):
        msg = SlackMessage(type="message",
                           user="U2147483697",
                           channel="C2147483705",
                           text="This is not the prefix you are looking for")
        yield from self.ping.process_message(msg)
        self.assertEqual(self.ping.send_version_message.mock_calls, [])

    def test_process_slack_message(self):
        msg = SlackMessage(type="message",
                           user="U2147483697",
                           channel="C2147483705",
                           text="!version")
        yield from self.ping.process_message(msg)
        expected = [call('C2147483705')]
        self.ping.send_version_message.assert_has_calls(expected)

    def test_process_pong_message(self):
        msg = SlackPong(type="pong",
                        reply_to="bob",
                        time="12:00am")
        yield from self.ping.process_message(msg)
        self.assertEqual(self.ping.send_version_message.mock_calls, [])



import asynctest
from asynctest.mock import call
from asynctest.mock import patch
from asynctest.mock import CoroutineMock


class TestGetRequest(asynctest.TestCase):

    def setUp(self):
        patcher = patch('charlesbot.util.http.aiohttp.get')
        from charlesbot.util.http import http_get_request
        self.http_get_request = http_get_request
        self.addCleanup(patcher.stop)
        self.mock_aiohttp_get = patcher.start()

    def test_non_200_response(self):
        response = CoroutineMock()
        response.status = 201
        response.text.return_value = "this is my return value"
        self.mock_aiohttp_get.side_effect = [response]
        retval = yield from self.http_get_request("https://www.example.com")  # NOQA
        expected = call('https://www.example.com',
                        headers={
                            'Content-type': 'application/json'
                        })
        self.assertEqual(self.mock_aiohttp_get.mock_calls, [expected])
        self.assertEqual(response.close.mock_calls, call())
        self.assertEqual(retval, "")

    def test_200_response(self):
        response = CoroutineMock()
        response.status = 200
        response.text.return_value = "this is my return value"
        self.mock_aiohttp_get.side_effect = [response]
        retval = yield from self.http_get_request("https://www.example.com")  # NOQA
        expected = call('https://www.example.com',
                        headers={
                            'Content-type': 'application/json'
                        })
        self.assertEqual(self.mock_aiohttp_get.mock_calls, [expected])
        self.assertEqual(response.close.mock_calls, [])
        self.assertEqual(retval, "this is my return value")



import yaml
import os


def read_config_dict(filename):
    with open(filename, 'r') as f:
        return yaml.load(f)


def get_config_file_name():
    env_var = "CHARLESBOT_SETTINGS_FILE"
    if env_var in os.environ:
        if os.environ[env_var]:
            return os.environ[env_var]
    return './development.yaml'


def get():
    config_dict = read_config_dict(get_config_file_name())
    return config_dict



from charlesbot.base_object import BaseObject
import json


class SlackAttachment(BaseObject):

    properties = ['color',
                  'fallback',
                  'text',
                  'mrkdwn_in',
                  'author_name',
                  'author_icon',
                  'thumb_url',
                  'title',
                  'title_link']

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    def __str__(self):
        return_dict = {}
        for prop in self.properties:
            return_dict.update({prop: getattr(self, prop)})
        return "[%s]" % json.dumps(return_dict)



from charlesbot.slack.slack_base_object import SlackBaseObject


class SlackMessage(SlackBaseObject):

    properties = ['type',
                  'channel',
                  'user',
                  'text',
                  'subtype']

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    @property
    def compatibility_key(self):
        return "message"



import unittest
from charlesbot.util.parse import parse_msg_with_prefix


class TestMessageParser(unittest.TestCase):

    def test_prefix_uppercase(self):
        msg = "!ALL hi, there!"
        retval = parse_msg_with_prefix("!all", msg)
        self.assertEqual("hi, there!", retval)

    def test_prefix_mixed(self):
        msg = "!AlL hi, there!"
        retval = parse_msg_with_prefix("!all", msg)
        self.assertEqual("hi, there!", retval)

    def test_prefix_colon(self):
        msg = "!all: hi, there!"
        retval = parse_msg_with_prefix("!all", msg)
        self.assertEqual("hi, there!", retval)

    def test_prefix_colon_two(self):
        msg = "!all:hi, there!"
        retval = parse_msg_with_prefix("!all", msg)
        self.assertEqual("hi, there!", retval)

    def test_prefix_space(self):
        msg = "!all hi, there!"
        retval = parse_msg_with_prefix("!all", msg)
        self.assertEqual("hi, there!", retval)

    def test_prefix_whitespace(self):
        msg = "!all  hi, there!"
        retval = parse_msg_with_prefix("!all", msg)
        self.assertEqual("hi, there!", retval)

    def test_prefix_leading_whitespace(self):
        msg = " !all  hi, there!"
        retval = parse_msg_with_prefix("!all", msg)
        self.assertEqual("hi, there!", retval)

    def test_prefix_leading_whitespace_two(self):
        msg = "  !all  hi, there!"
        retval = parse_msg_with_prefix("!all", msg)
        self.assertEqual("hi, there!", retval)

    def test_prefix_invalid_one(self):
        msg = "s  !all  hi, there!"
        retval = parse_msg_with_prefix("!all", msg)
        self.assertEqual(None, retval)

    def test_prefix_invalid_two(self):
        msg = "!allhi, there!"
        retval = parse_msg_with_prefix("!all", msg)
        self.assertEqual(None, retval)



import unittest
from charlesbot.util.slack import get_robot_group_membership
from tests import load_fixture


class TestGetRobotGroupMembership(unittest.TestCase):

    def setUp(self):
        self.group_list_one = load_fixture('group_list_one.json')
        self.group_list_two = load_fixture('group_list_two.json')
        self.group_list_three = load_fixture('group_list_three.json')
        self.group_list_four = load_fixture('group_list_four.json')

    def test_slack_group_membership_single(self):
        group_list = self.group_list_one
        membership = get_robot_group_membership(group_list)
        self.assertEqual(len(membership), 1)
        self.assertCountEqual(membership, {'G024BE91L': 'secretplans'})

    def test_slack_group_membership_none(self):
        group_list = "{}"
        membership = get_robot_group_membership(group_list)
        self.assertEqual(len(membership), 0)
        self.assertCountEqual(membership, {})

    def test_slack_group_membership_error_1(self):
        # malformed json
        group_list = self.group_list_two
        with self.assertRaises(ValueError):
            get_robot_group_membership(group_list)

    def test_slack_group_membership_error_2(self):
        # group key not present in json file
        group_list = self.group_list_three
        membership = get_robot_group_membership(group_list)
        self.assertEqual(len(membership), 0)
        self.assertCountEqual(membership, {})

    def test_slack_group_membership_multiple(self):
        # member of more than one group
        group_list = self.group_list_four
        membership = get_robot_group_membership(group_list)
        self.assertEqual(len(membership), 3)
        self.assertCountEqual(
            membership,
            {'G024BE91A': 'secretplans1',
             'G024BE91B': 'secretplans2',
             'G024BE91C': 'secretplans3'}
        )



import asynctest
import re
from asynctest.mock import MagicMock
from asynctest.mock import call
from charlesbot.plugins.help_plugin import Help
from charlesbot.slack.slack_message import SlackMessage


class TestHelpPlugin(asynctest.TestCase):

    def setUp(self):
        self.slack_client = MagicMock()
        self.hp = Help()
        self.hp.send_help_message = MagicMock()

    @asynctest.ignore_loop
    def test_help_message_wrapper_str(self):
        help_str = self.hp.get_help_message()
        regex = r'^```\n.*\n```$'
        p = re.compile(regex, re.MULTILINE | re.DOTALL)
        m = p.match(help_str)
        self.assertTrue(m)

    def test_wrong_message_type(self):
        msg = ""
        yield from self.hp.process_message(msg)
        self.hp.send_help_message.assert_has_calls([])

    def test_two_msgs_no_good(self):
        msg = SlackMessage(type="message",
                           user="U2147483697",
                           channel="C2147483705",
                           text="This is not me asking for prefix")
        yield from self.hp.process_message(msg)
        msg = SlackMessage(type="message",
                           user="U2147483697",
                           channel="C2147483705",
                           text="This is not me asking for help, either")
        yield from self.hp.process_message(msg)
        self.hp.send_help_message.assert_has_calls([])

    def test_two_msgs_one_good(self):
        msg = SlackMessage(type="message",
                           user="U2147483697",
                           channel="C1",
                           text="!help fine, now I need help")
        yield from self.hp.process_message(msg)
        msg = SlackMessage(type="message",
                           user="U2147483697",
                           channel="C2147483705",
                           text="This is not me asking for help, either")
        yield from self.hp.process_message(msg)
        expected = [call('C1')]
        self.hp.send_help_message.assert_has_calls(expected)

    def test_two_msgs_two_good(self):
        msg = SlackMessage(type="message",
                           user="U2147483697",
                           channel="C1",
                           text="!help fine, now I need help")
        yield from self.hp.process_message(msg)
        msg = SlackMessage(type="message",
                           user="U2147483697",
                           channel="C2",
                           text="!help Yep, still need help")
        yield from self.hp.process_message(msg)
        expected = [call('C1'), call('C2')]
        self.hp.send_help_message.assert_has_calls(expected, any_order=True)

    @asynctest.ignore_loop
    def test_help_msg_falsy(self):
        list_entry = "!help - This help message"
        self.hp.add_help_message("")
        self.assertEqual(len(self.hp.help_msg_list), 1)
        self.assertTrue(list_entry in self.hp.help_msg_list)
        self.hp.add_help_message([])
        self.assertEqual(len(self.hp.help_msg_list), 1)
        self.assertTrue(list_entry in self.hp.help_msg_list)
        self.hp.add_help_message(0)
        self.assertEqual(len(self.hp.help_msg_list), 1)
        self.assertTrue(list_entry in self.hp.help_msg_list)

    @asynctest.ignore_loop
    def test_add_help_msg(self):
        list_entry = "!help - This help message"
        self.assertEqual(len(self.hp.help_msg_list), 1)
        self.assertTrue(list_entry in self.hp.help_msg_list)
        self.hp.add_help_message("hi there!")
        self.assertEqual(len(self.hp.help_msg_list), 2)
        self.assertTrue(list_entry in self.hp.help_msg_list)
        self.assertTrue("hi there!" in self.hp.help_msg_list)



from setuptools import setup, find_packages
import codecs

# README into long description
with codecs.open('README.rst', encoding='utf-8') as f:
    readme = f.read()

setup(
    name='charlesbot',
    version='0.10.1',
    description='CharlesBOT is a Python bot written to take advantage of Slack\'s Real Time Messaging API',
    long_description=readme,
    author='Marvin Pinto',
    author_email='marvin@pinto.im',
    license='MIT',

   # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.4',
    ],
    keywords='slack robot chatops automation',

    url="https://github.com/marvinpinto/charlesbot",
    packages=find_packages(
        exclude=[
            'Makefile',
            'docker',
            'images',
            'requirements-dev.txt',
            'requirements.txt',
            'tests'
        ]
    ),

    entry_points={
        'console_scripts': ['charlesbot = charlesbot.__main__:main']
    },

    install_requires=[
        "slackclient==0.16",
        "websocket-client",
        "cchardet",
        "aiohttp",
        "PyYAML",
        "aiocron",
        "croniter",
        "python-dateutil",
    ],

    test_suite='nose.collector'
)



class Borg:
    _shared_state = {}

    def __init__(self):
        self.__dict__ = self._shared_state

    def _drop(self):
        "Drop the instance (for testing purposes)."
        self.__dict__ = {}



import logging
import asyncio
import signal
import functools
import importlib
from charlesbot.config import configuration
from charlesbot.slack.slack_connection import SlackConnection


class Robot(object):

    def __init__(self):  # pragma: no cover
        self.log = logging.getLogger(__name__)
        self.initialize_robot()

    def initialize_robot(self):  # pragma: no cover
        config_dict = configuration.get()
        self.token = config_dict['main']['slackbot_token']
        self.enabled_plugins = config_dict['main']['enabled_plugins']

    def is_running(self):
        return self._is_running

    def set_running(self, running):
        self._is_running = running

    @asyncio.coroutine
    def produce(self):
        while self.is_running():
            yield from self.route_message_to_plugin()

    @asyncio.coroutine
    def route_message_to_plugin(self):
        messages = yield from self.slack.get_stream_messages()
        for msg in messages:
            for plugin in self.plugin_list:
                yield from self.queue_message(msg, plugin)
        yield from asyncio.sleep(0.5)

    def initialize_plugins(self):
        return_list = []
        if not self.enabled_plugins:
            return return_list
        for x in self.enabled_plugins:
            module_name, class_name = x.rsplit(".", 1)
            obj_class = getattr(importlib.import_module(module_name), class_name)  # NOQA
            return_obj = obj_class()
            return_list.append(return_obj)
        return return_list

    def initialize_static_plugins(self):
        self.initialize_ping_plugin()
        self.initialize_help_plugin()

    def initialize_ping_plugin(self):
        from charlesbot.plugins.ping_plugin import Ping
        p = Ping()
        self.plugin_list.append(p)

    def initialize_help_plugin(self):
        from charlesbot.plugins.help_plugin import Help
        h = Help()
        for plugin in self.plugin_list:
            h.add_help_message(plugin.get_help_message())
        self.plugin_list.append(h)

    @asyncio.coroutine
    def queue_message(self, message, plugin):
        if message:
            yield from plugin.queue_message(message)

    def exit_cleanly(self):
        loop = asyncio.get_event_loop()
        self.log.info("Shutting down charlesbot")
        self.set_running(False)

        for plug in self.plugin_list:
            self.log.info("Shutting down plugin: %s" % plug.get_plugin_name())
            plug.is_running = False

        pending = asyncio.Task.all_tasks()
        asyncio.gather(*pending).cancel()
        loop.stop()

    def start(self):  # pragma: no cover
        self.set_running(True)
        self.slack = SlackConnection()
        loop = asyncio.get_event_loop()
        self.plugin_list = self.initialize_plugins()
        self.initialize_static_plugins()
        loop.create_task(self.produce())
        loop.add_signal_handler(
            signal.SIGINT,
            functools.partial(self.exit_cleanly)
        )
        try:
            loop.run_forever()
        finally:
            loop.close()



from charlesbot.util.borg import Borg
from slackclient import SlackClient
from charlesbot.config import configuration
import traceback
import asyncio
import json
import sys
import logging
import importlib


class SlackConnection(Borg):

    def __init__(self):
        super().__init__()
        self.log = logging.getLogger(__name__)
        self.initialized = False

    def initialize(self):  # pragma: no cover
        if not self.initialized:
            self.get_slack_token()
            self.connect()
            self.initialized = True

    def get_slack_token(self):  # pragma: no cover
        config_dict = configuration.get()
        self.token = config_dict['main']['slackbot_token']

    def connect(self):  # pragma: no cover
        self.sc = SlackClient(self.token)
        if not self.sc.rtm_connect():
            self.log.critical("Error conecting to Slack - token issue?")
            self.log.critical("--- full stack trace ---")
            self.log.critical(traceback.format_exc())
            sys.exit(1)

    @asyncio.coroutine
    def get_stream_messages(self):
        self.initialize()
        return_messages = []
        try:
            messages = self.sc.rtm_read()
            for msg in messages:
                message_object = self.get_message_type(msg)
                return_messages.append(message_object)
        except TimeoutError as t:
            self.log.error("Error reading from slack socket: %s" % t)
            self.log.error("--- full stack trace ---")
            self.log.error(traceback.format_exc())
            self.log.info("Now attempting to reconnect")
            self.connect()
        except BrokenPipeError as b:
            self.log.critical("Error reading from slack socket: %s" % b)
            self.log.critical("--- full stack trace ---")
            self.log.critical(traceback.format_exc())
            sys.exit(1)
        return return_messages

    def get_message_type(self, msg):
        self.initialize()
        obj_list = [
            "charlesbot.slack.slack_pong.SlackPong",
            "charlesbot.slack.slack_channel_joined.SlackChannelJoined",
            "charlesbot.slack.slack_channel_left.SlackChannelLeft",
            "charlesbot.slack.slack_group_joined.SlackGroupJoined",
            "charlesbot.slack.slack_group_left.SlackGroupLeft",
            "charlesbot.slack.slack_message.SlackMessage",
        ]
        for obj in obj_list:
            module_name, class_name = obj.rsplit(".", 1)
            obj_class = getattr(importlib.import_module(module_name), class_name)  # NOQA
            return_obj = obj_class()

            if getattr(return_obj, 'is_compatible')(msg):
                return_obj.load(msg)
                return return_obj
        return None

    @asyncio.coroutine
    def send_channel_message(self, channel_id, message):  # pragma: no cover
        self.initialize()
        self.sc.rtm_send_message(channel_id, message)

    @asyncio.coroutine
    def send_ping_message(self):  # pragma: no cover
        self.initialize()
        self.sc.server.ping()

    @asyncio.coroutine
    def api_call(self, api_endpoint, **kwargs):
        self.initialize()
        val = self.sc.api_call(api_endpoint, **kwargs)
        json_str = json.loads(val.decode('utf-8'))
        if not json_str['ok']:
            self.log.error(
                "Error fetching %s - response: %s",
                api_endpoint,
                str(json_str['ok'])
            )
            self.log.error(json_str)
            return json.dumps("{}")
        return json.dumps(json_str)



import asynctest
import json
from asynctest.mock import call
from asynctest.mock import patch


class TestSlackUser(asynctest.TestCase):

    def setUp(self):
        patcher1 = patch('charlesbot.slack.slack_connection.SlackConnection.api_call')  # NOQA
        self.addCleanup(patcher1.stop)
        self.mock_api_call = patcher1.start()

        from charlesbot.slack.slack_connection import SlackConnection
        self.slack_connection = SlackConnection()

        from charlesbot.slack.slack_user import SlackUser
        self.su = SlackUser()

    def tearDown(self):
        self.slack_connection._drop()

    @asynctest.ignore_loop
    def test_user_equality(self):
        from charlesbot.slack.slack_user import SlackUser
        user1 = SlackUser(id="SU01",
                          name="userone",
                          color="red")
        user2 = SlackUser(id="SU02",
                          name="usertwo",
                          color="blue")
        self.assertNotEqual(user1, user2)
        user2.id = "SU01"
        self.assertNotEqual(user1, user2)
        user2.name = "userone"
        self.assertNotEqual(user1, user2)
        user2.color = "red"
        self.assertEqual(user1, user2)

    @asynctest.ignore_loop
    def test_user_return_string(self):
        self.su.id = "SU01"
        self.su.name = "User One"
        self.su.deleted = False
        self.su.is_admin = False
        self.su.has_2fa = True
        user_json = json.loads(str(self.su))
        self.assertEqual(user_json.get('id'), "SU01")
        self.assertEqual(user_json.get('name'), "User One")
        self.assertEqual(user_json.get('deleted'), False)
        self.assertEqual(user_json.get('is_admin'), False)
        self.assertEqual(user_json.get('has_2fa'), True)
        self.assertEqual(user_json.get('is_owner'), "")

    def test_empty_slack_response(self):
        self.su.name = "suser"
        self.mock_api_call.side_effect = ["{}"]
        yield from self.su.retrieve_slack_user_info(self.slack_connection,
                                                    "fake123")
        expected_call = call("users.info", user="fake123")
        self.assertEqual(self.mock_api_call.mock_calls,
                         [expected_call]),
        self.assertEqual(self.su.name, "suser")
        self.assertEqual(self.su.last_name, "")
        self.assertEqual(self.su.is_bot, "")

    def test_no_profile_key(self):
        self.su.name = "suser"
        user_info = {
            "ok": True,
            "user": {
                "id": "U023BECGF",
                "name": "bobby"
            }
        }
        self.mock_api_call.side_effect = [json.dumps(user_info)]
        yield from self.su.retrieve_slack_user_info(self.slack_connection,
                                                    "fake123")
        expected_call = call("users.info", user="fake123")
        self.assertEqual(self.mock_api_call.mock_calls,
                         [expected_call]),
        self.assertEqual(self.su.name, "bobby")
        self.assertEqual(self.su.id, "U023BECGF")
        self.assertEqual(self.su.last_name, "")
        self.assertEqual(self.su.is_bot, "")

    def test_with_profile_key(self):
        self.su.name = "suser"
        user_info = {
            "ok": True,
            "user": {
                "id": "U023BECGF",
                "name": "bobby",
                "profile": {
                    "real_name": "Bobby Tables",
                    "image_24": "https://www.tables.com",
                }
            }
        }
        self.mock_api_call.side_effect = [json.dumps(user_info)]
        yield from self.su.retrieve_slack_user_info(self.slack_connection,
                                                    "fake123")
        expected_call = call("users.info", user="fake123")
        self.assertEqual(self.mock_api_call.mock_calls,
                         [expected_call]),
        self.assertEqual(self.su.name, "bobby")
        self.assertEqual(self.su.id, "U023BECGF")
        self.assertEqual(self.su.real_name, "Bobby Tables")
        self.assertEqual(self.su.image_24, "https://www.tables.com")
        self.assertEqual(self.su.is_bot, "")



from charlesbot.base_plugin import BasePlugin
from charlesbot.util.parse import does_msg_contain_prefix
from charlesbot.slack.slack_message import SlackMessage
from charlesbot.slack.slack_pong import SlackPong
import asyncio
from aiocron import crontab
import pkg_resources


class Ping(BasePlugin):

    def __init__(self):
        super().__init__("Ping Pong")
        self.schedule_ping_frequency()
        self.get_package_version_number()

    def get_package_version_number(self):  # pragma: no cover
        self.version = pkg_resources.require("charlesbot")[0].version

    def schedule_ping_frequency(self):  # pragma: no cover
        "Send a ping message to slack every 20 seconds"
        ping = crontab('* * * * * */20', func=self.send_ping, start=False)
        ping.start()

    def get_help_message(self):
        return "!version - List the running CharlesBOT version"

    @asyncio.coroutine
    def process_message(self, message):
        if not type(message) in [SlackMessage, SlackPong]:
            return

        if type(message) is SlackMessage:
            if does_msg_contain_prefix("!version", message.text):
                yield from self.send_version_message(message.channel)

        if type(message) is SlackPong:
            self.log.debug(str(message))

    @asyncio.coroutine
    def send_version_message(self, channel_id):  # pragma: no cover
        version_str = "CharlesBOT version %s" % self.version
        yield from self.slack.send_channel_message(channel_id,
                                                   version_str)

    @asyncio.coroutine
    def send_ping(self):  # pragma: no cover
        yield from self.slack.send_ping_message()
